/***************************************************************************
                          interceptor.cpp  -  description
                             -------------------
    begin                : lun avr 21 16:47:03 CEST 2003
    copyright            : (C) 2003 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstandarddirs.h>
#include <kapp.h>
#include <qpalette.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <kmessagebox.h>
#include "interceptor.h"
#include "itview.h"
#include "prefs.h"
#include "msg.h"
#include <qstringlist.h>
#include <cstdio>

extern const char *info_xpm[], *warn_xpm[], *fatal_xpm[];

ItView *MyItView, *RedirView;
MaList MaList;
extern const QString *Fptr;
QPixmap *PInfo, *PWarn, *PCrit;

#define FD(i) fifos[i].fd

extern "C"
{
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "pinfo.h"
#include "pwarn.h"
#include "pcrit.h"

  KPanelApplet* init( QWidget *parent, const QString configFile)
  {		
    KGlobal::locale()->insertCatalogue("interceptor");
    return new Interceptor(configFile, KPanelApplet::Normal,
                      KPanelApplet::About | /*KPanelApplet::Help |*/ KPanelApplet::Preferences,
                      parent, "interceptor");
  }
}

Interceptor::Interceptor(const QString& configFile, Type type, int actions, QWidget *parent, const char *name) 
       : KPanelApplet(configFile, type, actions, parent, name)
{
   // Get the current application configuration handle
   enum MessAction::Action act;
   int icon = 0;
   bool ok;
   QString args[5];
   ksConfig = config();
   NbLines = CurNbDB = 0;
   MaxNbDB = 3;
   setBackgroundMode(X11ParentRelative);
   //setFocusPolicy(StrongFocus);
   if(ksConfig->hasGroup("Rules"))
   {
		 int i = 0, j;
		 QStringList qsl;
		 QStringList::Iterator sit;
		 QString a;
		 ksConfig->setGroup("Rules");
		 while(1)
		 {
			 a.sprintf("Rule%d", i++);
			 qsl = ksConfig->readListEntry(a);
			 if(qsl.count() < 1) break;
			 args[0] = args[1] = args[2] = args[3] = args[4] = "";
			 for(j = 0, sit = qsl.begin(); sit != qsl.end() && j < 5; sit++, j++)
				 args[j] = *sit;
			 args[0] = args[0].lower();
			 if(args[0] == "beep")
				 act = MessAction::Beep;
       else if(args[0] == "redirect")
         act = MessAction::Redirect;
			else if(args[0] == "message")
				act = MessAction::Message;
			else
				act = MessAction::Run;
			icon = args[4].toInt(&ok);
			if(!ok)
				icon = 0;
      for(int k = 1; k < 4; k++)
        args[k].replace('\\', "\\\\");
			MaList.append(new MessAction(act, icon, args[1], args[2], args[3]));
		 }
	 }
   Cols[0].setNamedColor("green");
   Cols[1].setNamedColor("orange");
   Cols[2].setNamedColor("red");
   Cols[3] = colorGroup().background();
   PInfo = new QPixmap((const char **)pinfo);
   PWarn = new QPixmap((const char **)pwarn);
   PCrit = new QPixmap((const char **)pcrit);
   if(ksConfig->hasGroup("General"))
   {
		 ksConfig->setGroup("General");
		 Cols[0] = ksConfig->readColorEntry("Col0", &Cols[0]);
		 Cols[1] = ksConfig->readColorEntry("Col1", &Cols[1]);
		 Cols[2] = ksConfig->readColorEntry("Col2", &Cols[2]);
		 NbLines = ksConfig->readNumEntry("NbLines", 1);
     MaxNbDB = ksConfig->readNumEntry("MaxNbDialBox", 3);
	 }
   Icons[0] = new QPixmap(info_xpm);
   Icons[1] = new QPixmap(warn_xpm);
   Icons[2] = new QPixmap(fatal_xpm);
   Icons[3] = new QPixmap(getIcon("3floppy_unmount.png"));
   Icons[4] = new QPixmap(getIcon("cdrom_unmount.png"));
   Icons[5] = new QPixmap(getIcon("cdwriter_unmount.png"));
   Icons[6] = new QPixmap(getIcon("dvd_unmount.png"));
   Icons[7] = new QPixmap(getIcon("hdd_unmount.png"));
   Icons[8] = new QPixmap(getIcon("printer1.png"));
   Icons[9] = new QPixmap(getIcon("scanner.png"));
   Icons[10] = new QPixmap(getIcon("network.png", TRUE));
   Icons[11] = new QPixmap(getIcon("socket.png", TRUE));
   connect(&Kpr, SIGNAL(receivedStdout(KProcess*, char*, int)), this, SLOT(receivedStdout(KProcess*, char*, int)));
   IfVar.compile("\\$(1|2|3|4)");
   MyItView = new ItView(this);
   RedirView = new ItView(this, TRUE);
   aBtn = new Btn(this, MyItView, RedirView);
   It.setIt(this);
   It.start();
   aBtn->show();
}

QString Interceptor::getIcon(const QString& qs, bool fs)
{
	QString c = "kdeclassic/32x32/";
	c += (fs ? "filesystems/" : "devices/") + qs;
	c = KGlobal::dirs()->findResource("icon", c);
	if(c.isEmpty())
	{
		c = "hicolor/32x32/";
		c += (fs ? "filesystems/" : "devices/") + qs;
		c = KGlobal::dirs()->findResource("icon", c);
	}
	return c;
}

Interceptor::~Interceptor()
{
}

void Interceptor::about()
{
	KMessageBox::information(0, "Interceptor 0.8\nCopyright Christophe Gros 2003\nhttp://skylendar.kde.org/interceptor", "About");
}

void Interceptor::help()
{
	KApplication::kApplication()->invokeHelp();
}

void Interceptor::preferences()
{
   Prefs prefs(this);
   prefs.exec();  
}

int Interceptor::widthForHeight( int height ) const
{
	 aBtn->resize(20, height -2);
	 aBtn->repaint();
   return 22;
}

int Interceptor::heightForWidth( int width ) const
{
	 aBtn->resize(width - 2, 20);
	 aBtn->repaint();
   return 22;
}

void Interceptor::resizeEvent( QResizeEvent *_Event)
{
  QWidget::resizeEvent(_Event); 
}

void Interceptor::customEvent(QCustomEvent* ev)
{
	if(ev->type() != 1400)
		return;
	const LogLine* l = (const LogLine*)ev->data();
	MessAction* ma = l->Ma;
	switch(l->Level)
	{
		case Debug:
		case Info:
		case Notice: aBtn->IntButton->setPaletteBackgroundColor(Cols[0]); break;
		case Warning:
		case Err:    aBtn->IntButton->setPaletteBackgroundColor(Cols[1]); break;
		default:     aBtn->IntButton->setPaletteBackgroundColor(Cols[2]); break;
	}
	emit aBtn->IntButton->repaint();
	MyItView->getLine(l);
	qApp->processEvents(200);
	aBtn->IntButton->setPaletteBackgroundColor(Cols[3]);
	emit aBtn->IntButton->repaint();
	if(ma)
		switch(ma->action)
		{
			case MessAction::Beep:     beep(); break;
			case MessAction::Message:  message(*ma, l); break;
      case MessAction::Redirect: redirect(*ma, l); break;
			case MessAction::Run:      run(*ma, l); break;
			default:;
		}
	delete l;
}

void Interceptor::beep()
{
	qApp->beep();
}

void Interceptor::message(MessAction& ma, const LogLine* l)
{
	Msg* msg;
	bool yesno = TRUE;
	int ret;
	KProcess kpr;
	QString qs1 = ma.Arg1, qs2 = ma.Arg2;
	if(LastText == l->Text)
		return;
	else
		LastText = l->Text;
	if(CurNbDB == MaxNbDB) return;
	CurNbDB++;
	if(IfVar.match(qs1))
	{
		const QString p = Fptr[l->Level] + "##" + l->Source + "!!" + l->Host + "%%" + l->Text;
		{
			QMutexLocker qml(&MaList);
			ma.Kre.match(p.data());
			qs1.replace("$D", KGlobal::locale()->formatDate(l->Date, TRUE) );
			qs1.replace("$T", KGlobal::locale()->formatTime(l->Time) );
			qs1.replace("\\n", "\n");
			qs1.replace("$1", ma.Kre.group(1));
			qs1.replace("$2", ma.Kre.group(2));
			qs1.replace("$2", ma.Kre.group(3));
			qs1.replace("$3", ma.Kre.group(4));
			qs2.replace("$1", ma.Kre.group(1));
			qs2.replace("$2", ma.Kre.group(2));
			qs2.replace("$2", ma.Kre.group(3));
			qs2.replace("$3", ma.Kre.group(4));
		}
	}
	if(ma.Arg2.isEmpty())
		 yesno = FALSE;
	switch(l->Level)
		{
			case Debug:
			case Info:
			case Notice: msg = new Msg(ma.Icon > 0 ? *Icons[ma.Icon + FIRSTDEVICON - 1] : *Icons[0], qs1, i18n("Interceptor Info"), yesno); ret = msg->exec(); break;
			case Warning:
			case Err:    msg = new Msg(ma.Icon > 0 ? *Icons[ma.Icon + FIRSTDEVICON - 1] : *Icons[1], qs1, i18n("Interceptor Warning"), yesno); ret = msg->exec(); break;
		  default:     msg = new Msg(ma.Icon > 0 ? *Icons[ma.Icon + FIRSTDEVICON - 1] : *Icons[2], qs1, i18n("Interceptor Fatal"), yesno); ret = msg->exec(); break;
		};
		delete msg;
		CurNbDB--;
		LastText = "";
		if(yesno && ret == QDialog::Accepted)
		{
			kpr << qs2;
			kpr.start(KProcess::DontCare, KProcess::Stdout);
		}
}

void Interceptor::receivedStdout(KProcess *proc, char *buffer, int buflen)
{
	int i;
	char *c;
	QDate qd = QDate::currentDate();
	QTime qt = QTime::currentTime();
	for(c = buffer, i = 0; i < buflen && *c != '\n'; c++, i++);
	c = new char[i + 1];
	strncpy(c, buffer, i);
	c[i] = 0;
	QString r = c, so("Interceptor"), ho("localhost");
	const LogLine l(Info ,0, so, ho, r, qd, qt);
	MyItView->getLine(&l);
	delete c;
}

void Interceptor::run(MessAction& ma, const LogLine* l)
{
	QString qs1 = ma.Arg1, qs2 = ma.Arg2;
	if(IfVar.match(qs1))
	{
		const QString p = Fptr[l->Level] + "##" + l->Source + "!!" + l->Host + "%%" + l->Text;
		{
			QMutexLocker qml(&MaList);
			ma.Kre.match(p.data());
			qs1.replace("$D", KGlobal::locale()->formatDate(l->Date, TRUE) );
			qs1.replace("$T", KGlobal::locale()->formatTime(l->Time) );
			qs1.replace("\\n", "\n");
			qs1.replace("$1", ma.Kre.group(1));
			qs1.replace("$2", ma.Kre.group(2));
			qs1.replace("$2", ma.Kre.group(3));
			qs1.replace("$3", ma.Kre.group(4));
			qs2.replace("$1", ma.Kre.group(1));
			qs2.replace("$2", ma.Kre.group(2));
			qs2.replace("$2", ma.Kre.group(3));
			qs2.replace("$3", ma.Kre.group(4));
		}
	}
	if(!qs2.isEmpty())
	{
		if(!qs1.isEmpty()) MyItView->getLine(l);
		Kpr << qs2;
		Kpr.start(KProcess::DontCare, qs1.isEmpty() ? KProcess::Stdout : KProcess::NoCommunication);
	}
}

void Interceptor::redirect(MessAction& ma, const LogLine* l)
{
  RedirView->getLine(l);
  RedirView->show();
}
