/***************************************************************************
                          interthread.cpp  -  description
                             -------------------
    begin                : mer avr 23 2003
    copyright            : (C) 2003 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "interthread.h"
#include <qapp.h>
#include <qstringlist.h>
#include <klocale.h>
#include <qpixmap.h>
#include "interceptor.h"

extern "C"
{
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
};

extern QApplication* qApp;
extern QPtrList<MessAction> Mal;
extern MaList MaList;

static const char *MonName[] = { "Jan", "Feb", "Mar", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
static const QString Files[] = {"debug", "info", "notice", "warning", "err", "crit", "alert", "emerg" },
iFiles[] = { I18N_NOOP("Debug"), +I18N_NOOP("Info"), I18N_NOOP("Notice"), I18N_NOOP("Warning"),
I18N_NOOP("Error"), I18N_NOOP("Critical"), I18N_NOOP("Alert"), I18N_NOOP("Emergency") };
const QString *Fptr = Files, *iFptr = iFiles;

#define FD(i) fifos[i].fd

InterThread::InterThread()
{
	for(int i = 0; i < NBFILE; i++)
	{
		fifos[i].file =  "/var/run/interceptor/";
	  fifos[i].file	+= Files[i];
		fifos[i].level = (enum Levels)i;
		FD(i) = open(fifos[i].file.data(), O_RDONLY | O_NONBLOCK);
    fifos[i].f = fdopen(FD(i), "r");
	}
	LastText = " ";	
	Timeout.tv_sec = 50;
	Timeout.tv_usec = 0;
}

InterThread::~InterThread()
{
	for(int i = 0; i < NBFILE; i++)
    fclose(fifos[i].f);
		//close(FD(i));
}

void InterThread::run()
{
	int f;
	while(1)
	{
		max = -1;
		FD_ZERO(&Read);
		for(int i = 0; i < NBFILE; i++)
	  {
			if(FD(i) != -1)
			{
				FD_SET(FD(i), &Read);
				if(FD(i) > max)
					max = FD(i);
			}
		}
		if((f = select(max + 1, &Read, 0, 0, 0)) > 0)
		{
			for(int i = 0; i < NBFILE; i++)
				if(FD_ISSET(FD(i), &Read))
				{
          size_t s;
          char *c = 0;
          getdelim(&c, &s, '\n', fifos[i].f);
          for(char* c1 = c; *c1; c1++)
            if(*c1 == '\n') *c1 = ' ';
          if(strlen(c))
            process(c, i);
          free(c);
				}
		}
	}
}

void InterThread::process(const QString& qs, int i)
{
	QString	dayname, time, host, source, text;
	int mon = 1, day, year;
	MessAction *messa, *ma;
	QStringList qsl = QStringList::split(" ", qs);
	QStringList::Iterator it = qsl.begin();
	const QString monname = *it++;
	for(int j = 0; j < 12; j++)
	{
		if(strcmp(MonName[j], monname.data()) == 0)
		mon = j + 1;
	}
	dayname = *it++;
	time = *it++;
	QTime qt = QTime::fromString(time);
	day = dayname.toInt();
	QDate qd = QDate::currentDate();
	year = qd.year();
	qd.setYMD(year, mon, day);
	host = *it++;
	source = *it++;
	source.remove(':');
	while(it != qsl.end())
		text += *it++ + " ";
	{
		QMutexLocker qml(&MaList);
		const QString p = Files[i] + "##" + source + "!!" + host + "%%" + text;
		messa = 0;
		for(ma = MaList.first(); ma; ma = MaList.next())
		{
			if(ma->Kre.match(p.data()))
				messa = ma;
		}
	}
	LogLine* l = new LogLine(fifos[i].level, messa, source, host, text, qd, qt);
	QCustomEvent* qce = new QCustomEvent(1400);
	qce->setData(l);
	qApp->postEvent(QObj, qce);
}

MessAction::MessAction(enum Action a, int icon, const QString& pat, const QString& arg1, const QString& arg2)
{
	action = a;
	Icon = icon;
	Arg1 = arg1;
	PatStr = pat;
	Arg2 = arg2;
	Kre.compile(pat.data());
}

MaList::MaList()
{
}
