/***************************************************************************
                          interthread.h  -  description
                             -------------------
    begin                : mer avr 23 2003
    copyright            : (C) 2003 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INTERTHREAD_H
#define INTERTHREAD_H

#include <qthread.h>
#include <qevent.h>
#include <qmutex.h>
#include <qstring.h>
#include <qdatetime.h>
#include <kregexp.h>
#include <qptrlist.h>
#include <ctime>

extern "C"
{
#include <sys/select.h>
};

enum Levels { Debug, Info, Notice, Warning, Err, Crit, Alert, Emerg };

#define NBFILE 8

struct fifo
{
	QString file;
	int fd;
  FILE* f;
	enum Levels level;
};

class MessAction
{
public:

	enum Action { None, Beep, Message, Redirect, Run };

	MessAction(enum Action, int icon, const QString& pat = QString::null, const QString& arg1 = QString::null, const QString& arg2 = QString::null);
	enum Action action;
	int Icon;
	KRegExp Kre;
	QString PatStr, Arg1, Arg2;
};

class MaList : public QMutex, public QPtrList<MessAction>
{
public:

	MaList();
};

class LogLine
{
public:
	LogLine(enum Levels l, MessAction* ma, QString& source, QString& host, QString& text,
		QDate& date, QTime& time) { Level = l; Ma = ma; Source = source; Host = host; Text = text;
		Date = date; Time = time; }
	enum Levels Level;
	MessAction* Ma;
	QString Source, Host, Text;
	QDate Date;
	QTime Time;
};

/**this class manages the syslog fifo processing and filtering
  *@author Christophe Gros
  */

class InterThread : public QThread  {
public: 
	InterThread();
	void setIt(QObject* qobj) { QObj = qobj; }
	~InterThread();

	void run();

private:

void process(const QString&, int);
fd_set Read;
struct timeval Timeout;
struct fifo fifos[NBFILE];
int max, min;
QObject* QObj;
QString LastText;	
};

#endif
